import matplotlib as mpl
from ._setup import setup
from ._format import add_colorbar, format_datetime_axis
from ._draw import draw_arrows, draw_earth, draw_multicolored_line


mplc = mpl.colors
plt = mpl.pyplot
setup()
